{Hanoi 1.1
===============================================================
Torres de Hanoi es un divertido juego de inteligencia cuya misin es pasar en la menor
cantidad de movimientos posibles los aros de la columna de la izquierda a la columna de la
derecha usando la columna del centro como ayuda pero sin colocar aros grandes sobre pequeos.
Para hacerlo se le pedir el nmero de la columna donde est el aro que desea mover y luego el
nmero de columna al que desea mover el aro.

Ejemplo:

    Origen:   1
    Destino:  3

    Jugadas: 0

                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                 ___                  |                   |
                _____                 |                   |
               _______                |                   |
    _______________________________________________________________________
                  1                   2                   3

Mover el primer aro de la columna 1 y lo colocar en la columna dos, el resultado ser el
siguiente:


    Origen:
    Destino:

    Jugadas: 1

                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                  |                   |                   |
                _____                 |                   |
               _______                |                  ___
    _______________________________________________________________________
                  1                   2                   3




Cdigo fuente
===============================================================
Contiene el cdigo fuente del programa (compilado con Turbo Pascal 5.5 (MS-DOS) y
FreePascal (LINUX) ): Hanoi.pas es el programa principal y TPUtils.pas son
utilidades variadas para Pascal que este programa usa.

Licencia:
===============================================================
Este software es dado "tal-cual" sin ningn tipo de garanta, el autor no se responsabiliza por
daos ocasionados por el uso de este programa.

Se autoriza a cualquier persona distribuir y modificar el cdigo fuente y el programa compilado
 bajo las siguientes condiciones:

   1.  Versiones modificadas deben ser tratadas como tales y no como el programa original,
       usted no debe decir que escribi el programa original.
   2.  Este aviso no debe ser quitado ni alterado de ninguna manera.

===============================================================
(C) 2002-2004 Hctor Mauricio Rodrguez Segura.}
Program Hanoi;

{$ifdef FPC}
   {$mode TP}
{$endif}


uses
  Crt, TPUtils;

Type
  TRosca = 0..9;
  TCol = Array[1..10] of TRosca;
  TColumnas = Array[1..3] of TCol;

Const
  ColChar = '|';
{$ifdef LINUX}
  { Depende de la pagina de codigos que se use el caracter ASSI220 no se vera
    bien para representar los aros}
  RoscaChar = '=';
{$else}
  { En DOS el ANSI220 siempre es lo mismo }
  RoscaChar = #220;
{$endif}
  ColsX = (80 - 42) div 2;
  ColsY = 20;

  BaseColor = LightBlue;
  RoscaColor = Yellow;
  ColColor = Red;

  DelaySound = 1000;


  OSID = {$ifdef LINUX}'Linux'{$else}'MS-DOS'{$endif};

var
  Cols: TColumnas;
  Aros: TRosca;
  Jugadas: Integer;
  

Procedure InitCols(R: TRosca);
var
  A, X, Y: Integer;
Begin
  A := R;
  Jugadas := 0;
  For X := 1 to 3 do
    For Y := 1 to 10 do
    Begin
      if (X = 1) and (Y <= R) then
      Begin
       Cols[X][Y] := A;
       Dec(A);
      end else
        Cols[X][Y] := 0;
    end;
end;

Function Ultima(Col: Byte): Byte;
var
  R: Byte;
Begin
  For R := 10 Downto 1 do
  if Cols[Col][R] > 0 then
  Begin
    Ultima := R;
    Exit;
  end;
  Ultima := 0;
end;

Procedure Mover(Src, Dest: Byte);
var
  C: Integer;

Var
  SrcIdx, DestIdx: Byte;
  Camote: Char;
Begin
  SrcIdx := Ultima(Src);
  DestIdx := Ultima(Dest);
  if SrcIdx = 0 then
  Begin
    Camote := ShowMsg('La columna de origen est basida.', True,
      500, DelaySound);
  end else
  if (Cols[Src][SrcIdx] < Cols[Dest][DestIdx]) or
   (Cols[Dest][DestIdx] = 0) then
  Begin
    Cols[Dest][DestIdx + 1] := Cols[Src][SrcIdx];
    Cols[Src][SrcIdx] := 0;
    Inc(Jugadas);
  end else
  if SrcIdx <> DestIdx then
  Begin
    Camote := ShowMsg('Aro grande sobre pequeo.', True,
      500, DelaySound);
  end else
  Begin
    Camote := ShowMsg('La columna de origen es la misma que la de destino.',
      True, 500, DelaySound);
  end;
end;

Procedure DibujarPantalla;
var
  C, I, X, X2, Y: Integer;
  R: TRosca;
Begin
  ClrScr;
  TextColor(White);
  Centrar('Torres de Hanoi (' + OSID + ') - Hector Mauricio Rodriguez Segura', 1);
  TextColor(Red);
  Centrar(StringOfChar('-', 60), 2);
  TextBackGround(White); TextColor(Black);
  Centrar(' Alt+X Salir - F2 Reiniciar - F3 Seleccionar cantidad de aros ', 3);
  TextBackGround(Black);
  TextColor(Blue);
  WriteXY(5, 6, 'Origen:');
  WriteXY(5, 7, 'Destino:');
  WriteXY(5, 9, 'Jugadas: '+ IntToStr(Jugadas));

  X := ColsX;
  TextColor(BaseColor);
  For I := 5 to 75 do
  Begin
    GotoXY(I, ColsY + 1);
    Write(RoscaChar);
  end;
  For I := 1 to 3 do
  Begin
    Y := ColsY;
    For C := 1 to 10 do
    Begin
      R := Cols[I][C];
      if R = 0 then
      Begin
        TextColor(ColColor);
        GotoXY(X, Y);
        Write(ColChar);
      end else
      Begin
        For X2 := X - R to X + R do
        Begin
          TextColor(RoscaColor);
          GotoXY(X2, Y);
          Write(RoscaChar);
        end;
      end;
      TextColor(BaseColor);
      WriteXY(X, ColsY + 2, IntToStr(X div ColsX));
      Dec(Y);
    end;
    Inc(X, 20);
  end;
end;

Procedure LeerMovimiento;
Var
  C: Char;
  A, B: Byte;
  E: Integer;
  
  Function ReadMov: Byte;
  Var
    R: Byte;
  Begin
    C := ReadChar;
    ReadMov := 0;
    if CharAlfaNum then
    Begin
      if C in ['1'..'3'] then
      Begin
        Val(C, R, E);
        Write(C);
        ReadMov := R;
      end else
      Begin
        Sound(1000, 100);
        ReadMov := ReadMov;
      end;
    end else
    Case C of
      AltX: Halt;
      F2: Begin
            InitCols(Aros);
            Jugadas := 0;
          end;
      F3: Begin
            Repeat
              C := ShowMsg('Nmero de aros: [3..9]', True, 500, 100);
            Until C in ['3'..'9', #27];
            if C <> #27 then
            Begin
              Val(C, Aros, E);
              InitCols(Aros);
              Jugadas := 0;
            end;
          end;
      end;
  end;

Begin
  GotoXY(15, 6); A := ReadMov;
  if A = 0 then Exit;
  GotoXY(15, 7); B := ReadMov;
  if B = 0 then Exit;
  Mover(A, B);
  if Ultima(3) = Aros then
  Begin
    DibujarPantalla;
    C := ShowMsg('Felicidades lo lograste!!!', True, 500, 100);
    InitCols(Aros);
  end;
end;

Var
  Op: Char;
  A, B: Byte;
  SaveTextAttr, SaveTextMode: Integer;
  SaveExitProc: Pointer;
  
 {$F+} 
 procedure DeInit; 
 begin
   ExitProc := SaveExitProc; 
   TextAttr := SaveTextAttr;
   TextMode(SaveTextMode);
   ClrScr;
 end;
 {$F-}
 
 procedure Init;
 begin
   SaveTextAttr := TextAttr;
   SaveTextMode := LastMode;
   TextMode(CO80);
   SaveExitProc := ExitProc;
   ExitProc := @DeInit;  
   if Aros = 0 then
     Aros := 3;
   InitCols(Aros);
 end;
 
 
procedure CheckParams;
Var
  I, E: Integer;
begin
  if ParamStr(1) = '-a' then
  begin
    Val(ParamStr(2), I, E);
    if (E = 0) and (I in [3..9]) then
       Aros := I;
  end else
  if ParamStr(1) = '--help' then
  begin
    Writeln('Torres de Hanoi por Hector Mauricio Rodriguez Segura (c) 2004');
    Writeln('Uso:');
    Writeln('  ' + ParamStr(0) + ' [-a {3..9}]');
    Halt;
  end;

end;
Begin
  CheckParams;
  Init;
  Repeat
    DibujarPantalla;
    LeerMovimiento;
  Until False;
end.
